// GPRS.H
//
// Copyright (c) 1997-1999 Symbian Ltd.  All rights reserved.
//

#if !defined(__ETELGPRS_H__)
#define __ETELGPRS_H__

#include <e32base.h>
#include <s32mem.h>
#include <ETEL.h>

#include <exterror.h>

//
//  Global Gprs constants and types
// 

const TInt KETelExtGprsV1 = 4000;	// For passing into RTelServer::IsSupportedByModule()


const TInt KErrGprsServicesNotAllowed=KErrGsmMobilityManagementBase-7;
const TInt KErrGprsAndNonGprsServicesNotAllowed=KErrGsmMobilityManagementBase-8;
const TInt KErrGprsMSIdentityCannotBeDerivedByTheNetwork=KErrGsmMobilityManagementBase-9;
const TInt KErrGprsMSImplicitlyDetached=KErrGsmMobilityManagementBase-10;
const TInt KErrGprsMSCTemporarilyNotReachable=KErrGsmMobilityManagementBase-16;
const TInt KErrGprsLlcOrSndcpFailure=KErrGsmMobilityManagementBase-25;
const TInt KErrGprsInsufficientResources=KErrGsmMobilityManagementBase-26;
const TInt KErrGprsMissingorUnknownAPN=KErrGsmMobilityManagementBase-27;
const TInt KErrGprsUnknownPDPAddress=KErrGsmMobilityManagementBase-28;
const TInt KErrGprsUserAuthenticationFailure=KErrGsmMobilityManagementBase-29;
const TInt KErrGprsActivationRejectedByGGSN=KErrGsmMobilityManagementBase-30;
const TInt KErrGprsActivationRejected=KErrGsmMobilityManagementBase-31;
const TInt KErrGprsNSAPIAlreadyUsed=KErrGsmMobilityManagementBase-35;
const TInt KErrGprsRegularContextDeactivation=KErrGsmMobilityManagementBase-36;
const TInt KErrGprsQoSNotAccepted=KErrGsmMobilityManagementBase-37;	
const TInt KErrGprsReactivationRequested=KErrGsmMobilityManagementBase-39;
const TInt KErrGprsFeatureNotSupported=KErrGsmMobilityManagementBase-40;

enum TPdpType
	{
	EPdpTypeIPv4,
	EPdpTypeIPv6,
	EPdpTypeX25
	};


class RGprs;
class CGprsContextPtrHolder;
class RGprsContext : public RTelSubSessionBase
	{
public:
	IMPORT_C RGprsContext();
	IMPORT_C TInt OpenNewContext(RGprs& aGprs,TDes& aContextName);
	IMPORT_C TInt OpenExistingContext(RGprs& aGprs,const TDesC& aContextName);
	IMPORT_C void Close();

	enum {KGSNNameLength = 252};
	enum {KMaxPDPAddressLength = 20};
	enum 
		{
		KPdpDataCompression		= 0x01,
		KPdpHeaderCompression	= 0x02
		};

	enum TQoSPrecedencePhase1
		{
		ESubscribedPrecedence,
		EPriorityHigh,
		EPriorityMedium,
		EPriorityLow
		};

	enum TQoSDelayPhase1
		{
		ESubscribedDelayClass,
		EDelayClass1,
		EDelayClass2,
		EDelayClass3,
		EDelayClass4
		};

	enum TQoSReliabilityPhase1
		{
		ESubscribedReliabilityClass,
		EReliabilityClass1,
		EReliabilityClass2,
		EReliabilityClass3,
		EReliabilityClass4,
		EReliabilityClass5
		};

	enum TQoSPeakThroughputPhase1
		{
		ESubscribedPeakThroughput,
		EPeakThroughput1000,
		EPeakThroughput2000,
		EPeakThroughput4000,
		EPeakThroughput8000,
		EPeakThroughput16000,
		EPeakThroughput32000,
		EPeakThroughput64000,
		EPeakThroughput128000,
		EPeakThroughput256000
		};

	enum TQoSMeanThroughputPhase1
		{
		ESubscribedMeanThroughput,	
		EMeanThroughputBestEffort,
		EMeanThroughput100,
		EMeanThroughput200,
		EMeanThroughput500,
		EMeanThroughput1000,
		EMeanThroughput2000,
		EMeanThroughput5000,
		EMeanThroughput10000,
		EMeanThroughput20000,
		EMeanThroughput50000,
		EMeanThroughput100000,
		EMeanThroughput200000,
		EMeanThroughput500000,
		EMeanThroughput1000000,
		EMeanThroughput2000000,
		EMeanThroughput5000000,
		EMeanThroughput10000000,
		EMeanThroughput20000000,
		EMeanThroughput50000000
		};

	struct TQoSPhase1
		{
		TQoSPrecedencePhase1 iPrecedence;
		TQoSDelayPhase1 iDelay;
		TQoSReliabilityPhase1 iReliability;
		TQoSPeakThroughputPhase1 iPeakThroughput;
		TQoSMeanThroughputPhase1 iMeanThroughput;
		};

	enum TAnonymousAccess
		{
		ENotApplicable,
		ERequired,
		ENotRequired
		};

	typedef TBuf<KGSNNameLength> TGSNAddress;
	typedef TBuf<KMaxPDPAddressLength> TPdpAddress;

	class TContextConfigBase
		{
	public:
		inline TInt ExtensionId(){return iExtensionId;};
	protected:
		TInt iExtensionId;
		};

	enum 
		{
		KContextConfigPhase1= 0x01
		};

	class TContextConfigV01 : public TContextConfigBase
		{
	public:
		IMPORT_C TContextConfigV01();
	public:
		TPdpType iPdpType;
		TGSNAddress iAccessPointName;
		TPdpAddress iPdpAddress;		
		TUint iPdpCompression;
		TQoSPhase1 iQoSRequested;
		TQoSPhase1 iQoSMinimum;
		TAnonymousAccess iAnonymousAccessReqd;
		};
	
	typedef TPckg<TContextConfigV01> TContextConfigPckg;

	IMPORT_C TInt SetConfig(const TDesC8& aConfig) const;
	IMPORT_C void SetConfig(TRequestStatus& aStatus, const TDesC8& aConfig) const;
	IMPORT_C void SetConfigCancel() const;

	IMPORT_C TInt GetConfig(TDes8& aConfig) const;
	IMPORT_C void GetConfig(TRequestStatus& aStatus, TDes8& aConfig) const;
	IMPORT_C void GetConfigCancel() const;

	IMPORT_C void NotifyConfigChanged(TRequestStatus& aStatus, TDes8& aConfig) const;
	IMPORT_C void NotifyConfigChangedCancel() const;

	IMPORT_C TInt Activate() const;
	IMPORT_C void Activate(TRequestStatus& aStatus) const;
	IMPORT_C void ActivateCancel() const;

	IMPORT_C TInt Deactivate() const;
	IMPORT_C void Deactivate(TRequestStatus& aStatus) const;
	IMPORT_C void DeactivateCancel() const;

	IMPORT_C TInt Delete() const;
	IMPORT_C void Delete(TRequestStatus& aStatus) const;
	IMPORT_C void DeleteCancel() const;

	IMPORT_C TInt LoanCommPort(RCall::TCommPort& aDataPort) const;
	IMPORT_C void LoanCommPort(TRequestStatus& aStatus, RCall::TCommPort& aDataPort) const;
	IMPORT_C void LoanCommPortCancel() const;
	IMPORT_C TInt RecoverCommPort() const;
	IMPORT_C void RecoverCommPort(TRequestStatus& aStatus) const;
	IMPORT_C void RecoverCommPortCancel() const;

	enum TContextStatus
		{
		EStatusUnknown,
		EStatusInactive,
		EStatusActivating,
		EStatusActive,
		EStatusDeactivating,		
		EStatusSuspended,
		EStatusDeleted
		};

	IMPORT_C TInt GetStatus(TContextStatus& aContextStatus) const;
	IMPORT_C void NotifyStatusChange(TRequestStatus& aStatus,TContextStatus& aContextStatus);
	IMPORT_C void NotifyStatusChangeCancel() const;

	struct TDataVolume 
		{
		TUint32 iBytesSent;
		TUint32 iOverflowCounterSent;
		TUint32 iBytesReceived;
		TUint32 iOverflowCounterReceived;
		};

	struct TNotifyDataTransferredRequest
		{
		TUint iRcvdGranularity;
		TUint iSentGranularity;
		};

	IMPORT_C TInt GetDataVolumeTransferred(TDataVolume& aVolume) const; 
	IMPORT_C void NotifyDataTransferred(TRequestStatus& aStatus, TDataVolume& aVolume, TUint aRcvdGranularity=0x1000,TUint aSentGranularity=0x1000) const; 
	IMPORT_C void NotifyDataTransferredCancel() const; 

	IMPORT_C TInt GetLastErrorCause(TInt& aError) const;
private:
	CGprsContextPtrHolder* iEtelGprsPtrHolder;
protected:
	IMPORT_C virtual void ConstructL();
	IMPORT_C virtual void Destruct();
	};

//
// RGprs
//
class CGprsPtrHolder;
class RGprs : public RTelSubSessionBase
	{
public:
	IMPORT_C RGprs();
	IMPORT_C TInt Open(RPhone& aPhone);
	IMPORT_C void Close();

	IMPORT_C void NotifyContextAdded(TRequestStatus& aStatus, TDes& aContextId) const;
	IMPORT_C void NotifyContextAddedCancel() const;

	IMPORT_C TInt Attach() const;	
	IMPORT_C void Attach(TRequestStatus& aStatus) const;	
	IMPORT_C void AttachCancel() const;	

	IMPORT_C TInt Detach() const;	
	IMPORT_C void Detach(TRequestStatus& aStatus) const;	
	IMPORT_C void DetachCancel() const;	

	enum TStatus
		{
		EStatusUnattached,
		EStatusAttached,
		EStatusActive,
		EStatusSuspended
		};


	IMPORT_C TInt GetStatus(TStatus& aGprsStatus) const;
	IMPORT_C void NotifyStatusChange(TRequestStatus& aStatus,TStatus& aGprsStatus) const;
	IMPORT_C void NotifyStatusChangeCancel() const;

	IMPORT_C void NotifyContextActivationRequested(TRequestStatus& aStatus, TPdpType& aPdpType, RGprsContext::TPdpAddress& aPdpAddress) const;
	IMPORT_C void NotifyContextActivationRequestedCancel() const;

	IMPORT_C TInt RejectActivationRequest() const;
	IMPORT_C void RejectActivationRequest(TRequestStatus& aStatus) const;
	IMPORT_C void RejectActivationRequestCancel() const;

	struct TContextInfo
		{
		TName iName;
		RGprsContext::TContextStatus iStatus;
		};

	IMPORT_C TInt EnumerateContexts(TInt& aCount, TInt& aMaxAllowed) const;
	IMPORT_C TInt GetContextInfo(TInt aIndex,TContextInfo& aInfo) const;
	IMPORT_C void GetContextInfo(TRequestStatus& aStatus,TInt aIndex,TContextInfo& aInfo) const;
	IMPORT_C void GetContextInfoCancel() const;

	//
	//	network reg stuff
	//
	enum TRegistrationStatus
		{
		ENotRegisteredNotSearching,
		ERegisteredOnHomeNetwork,
		ENotRegisteredSearching,
		ERegistrationDenied,
		EUnknown,
		ERegisteredRoaming,
		ENotRegisteredButAvailable,	
		ENotRegisteredAndNotAvailable
		};

	IMPORT_C TInt GetNtwkRegStatus(TRegistrationStatus& aRegistrationStatus) const;
	IMPORT_C void NotifyChangeOfNtwkRegStatus(TRequestStatus& aStatus,TRegistrationStatus& aRegistrationStatus) const;
	IMPORT_C void NotifyChangeOfNtwkRegStatusCancel() const;

	enum TMSClass
		{
		EMSClassA,
		EMSClassB,
		EMSClassCAlternateMode,
		EMSClassCGPRSOnly,
		EMSClassCCircuitSwitchedOnly,
		EMSClassUnknown
		};

	IMPORT_C TInt GetMSClass(TMSClass& aCurrentClass, TMSClass& aMaxClass) const;
	IMPORT_C void GetMSClass(TRequestStatus& aStatus, TMSClass& aCurrentClass, TMSClass& aMaxClass) const;
	IMPORT_C void GetMSClassCancel() const;

	IMPORT_C TInt SetMSClass(TMSClass aClass) const;
																						
	IMPORT_C void SetMSClass(TRequestStatus& aStatus, TMSClass aClass) const;
	IMPORT_C void SetMSClassCancel() const;

	IMPORT_C void NotifyMSClassChange(TRequestStatus& aStatus, TMSClass& aNewClass) const;
	IMPORT_C void NotifyMSClassChangeCancel() const;

	//
	// GPRS Capabilities
	//
	enum TQoSPrecedenceCaps
		{
		KCapsPrecedenceHigh		= 0x01,
		KCapsPrecedenceMedium	= 0x02,
		KCapsPrecedenceLow		= 0x04
		};

	enum TQoSDelayCaps
		{
		KCapsDelayClass1 = 0x01,
		KCapsDelayClass2 = 0x02,
		KCapsDelayClass3 = 0x04,
		KCapsDelayClass4 = 0x08
		};

	enum TQoSReliabilityCaps
		{
		KCapsReliabilityClass1 = 0x01,
		KCapsReliabilityClass2 = 0x02,
		KCapsReliabilityClass3 = 0x04,
		KCapsReliabilityClass4 = 0x08,
		KCapsReliabilityClass5 = 0x10
		};

	enum TQoSPeakThrghptCaps
		{
		KCapsPeak1000 = 0x001,
		KCapsPeak2000 = 0x002,		
		KCapsPeak4000 = 0x004,		
		KCapsPeak8000 = 0x008,		
		KCapsPeak16000 = 0x010,	
		KCapsPeak32000 = 0x020,	
		KCapsPeak64000 = 0x040,	
		KCapsPeak128000 = 0x080,	
		KCapsPeak256000 = 0x100
		};

	enum TQoSMeanThrghptCaps
		{
		KCapsMean100		= 0x00001,
		KCapsMean200		= 0x00002,		
		KCapsMean500		= 0x00004,		
		KCapsMean1000		= 0x00008,		
		KCapsMean2000		= 0x00010,	
		KCapsMean5000		= 0x00020,	
		KCapsMean10000		= 0x00040,	
		KCapsMean20000		= 0x00080,	
		KCapsMean50000		= 0x00100,
		KCapsMean100000		= 0x00200,
		KCapsMean200000		= 0x00400,
		KCapsMean500000		= 0x00800,
		KCapsMean1000000	= 0x01000,
		KCapsMean2000000	= 0x02000,
		KCapsMean5000000	= 0x04000,
		KCapsMean10000000	= 0x08000,
		KCapsMean20000000	= 0x10000,
		KCapsMean50000000	= 0x20000,
		};

	struct TQoSCaps
		{
		TUint iPrecedence;
		TUint iDelay;
		TUint iReliability;
		TUint iPeak;
		TUint iMean;
		};
	
	enum TStaticMiscCaps
		{
		KCapsSuspendSupported=0x0001,
		KCapsAASupported=0x0002,
		KCapsNetworkAvailabilitySupported=0x0004,
		KCapsSetDefaultContextSupported=0x0008,
		KCapsChangeAttachModeSupported=0x0010,
		KCapsGetDataTransferredSupported=0x0020,
		KCapsNotifyDataTransferredSupported=0x0040,
		KCapsPreferredBearerSupported=0x0080,
		KCapsPdpDataCompSupported=0x0100,
		KCapsPdpHeaderCompSupported=0x0200,
		KCapsMSClassSupported=0x0400,
		KCapsNotifyMSClassSupported=0x0800
		};

	class TStaticCapsV01
		{
	public:
		TQoSCaps iQosCaps;
		TUint iMiscCaps;
		};
	typedef TPckg<TStaticCapsV01> TStaticCaps;

	IMPORT_C TInt GetStaticCaps(TDes8& aCaps,TPdpType aPdpType=EPdpTypeIPv4) const;

	enum TDynamicCaps
		{
		KCapsAttach=0x00000001,
		KCapsDetach=0x00000002,
		KCapsActivate=0x00000004,
		KCapsRxCSCall=0x00000008,
		KCapsRxContextActivationReq=0x00000010,
		KCapsSMSTransfer=0x00000020
		};
	
	typedef TUint TDynamicCapsFlags;

	IMPORT_C TInt GetDynamicCaps(TDynamicCapsFlags& aCaps) const;
	IMPORT_C void NotifyDynamicCapsChange(TRequestStatus& aStatus, TDynamicCapsFlags& aCaps) const;
	IMPORT_C void NotifyDynamicCapsChangeCancel() const;

	//
	//	SMS over GPRS. 
	//

	enum TSmsBearer
		{
		ESmsBearerGprsOnly,
		ESmsBearerCircuitSwitchedOnly,
		ESmsBearerGprsPreferred,
		ESmsBearerCircuitSwitchedPreferred		
		};

	IMPORT_C TInt SetSmsBearer(TSmsBearer aBearer) const;
	IMPORT_C void SetSmsBearer(TRequestStatus& aStatus, TSmsBearer aBearer) const;
	IMPORT_C void SetSmsBearerCancel() const;

	IMPORT_C TInt GetSmsBearer(TSmsBearer& aBearer) const;
	IMPORT_C void GetSmsBearer(TRequestStatus& aStatus, TSmsBearer& aBearer) const;
	IMPORT_C void GetSmsBearerCancel() const;

	IMPORT_C void NotifySmsBearerChange(TRequestStatus& aStatus, TSmsBearer& aNewBearer) const;
	IMPORT_C void NotifySmsBearerChangeCancel() const;

	enum TPreferredBearer
		{
		EBearerGprs,
		EBearerCircuitSwitched
		};

	IMPORT_C TInt SetPreferredBearer(TPreferredBearer aBearer) const;
	IMPORT_C void SetPreferredBearer(TRequestStatus& aStatus, TPreferredBearer aBearer) const;
	IMPORT_C void SetPreferredBearerCancel() const;
	IMPORT_C TInt GetPreferredBearer(TPreferredBearer& aBearer) const;

	enum TAttachMode
		{
		EAttachWhenPossible,
		EAttachWhenNeeded
		};

	IMPORT_C TInt SetAttachMode(TAttachMode aMode) const;
	IMPORT_C TInt GetAttachMode(TAttachMode& aMode) const;

	IMPORT_C TInt SetDefaultContextParams(const TDesC8& aPckg) const;
	IMPORT_C TInt GetDefaultContextParams(TDes8& aPckg) const;
private:
	CGprsPtrHolder* iEtelGprsPtrHolder;
protected:
	IMPORT_C virtual void ConstructL();
	IMPORT_C virtual void Destruct();
	};

#endif

